/* audioY */

#include <stdio.h>
#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/device.h>
#include <zephyr/storage/disk_access.h>
#include <zephyr/logging/log.h>
#include <zephyr/fs/fs.h>

/* Note the fatfs library is able to mount only strings inside _VOLUME_STRS in ffconf.h */
#if defined(CONFIG_FAT_FILESYSTEM_ELM)
#include <ff.h>

#define DISK_DRIVE_NAME "/SD:"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

static FATFS fat_fs;
// static struct fs_file_t file;

/* mounting info */
static struct fs_mount_t mp = {
	.type = FS_FATFS,
	.mountp_len = DISK_MOUNT_PT,	// orgiginally .mountp
	.fs_data = &fat_fs,
};

#elif defined(CONFIG_FILE_SYSTEM_EXT2)

#include <zephyr/fs/ext2.h>
#define DISK_DRIVE_NAME "SDMMC"
#define DISK_MOUNT_PT "/ext"

static struct fs_mount_t mp = {
	.type = FS_EXT2,
	.flags = FS_MOUNT_FLAG_NO_FORMAT,
	.storage_dev = (void *)DISK_DRIVE_NAME,
	.mnt_point = "/ext",
};

/* little FS is what is enabled here, may need to change to ext2, fat, nffs, or other */
#elif defined(CONFIG_FILE_SYSTEM_NFFS)
#include <nffs/nffs.h>
#define DISK_DRIVE_NAME "/nffs"	// change DISK_DRIVE_NAME to "/nffs:" to match ELM above ?
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#elif defined(CONFIG_FILE_SYSTEM_LITTLEFS)
#include <fs/littlefs.h>
#define DISK_DRIVE_NAME "/lfs"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#elif defined(CONFIG_FILE_SYSTEM_NATIVE_POSIX)
#define DISK_DRIVE_NAME "/disk"	// originally DISK_MOUNT_PT "/disk"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#elif defined(CONFIG_FILE_SYSTEM_MCUBFS)
#include <fs/mcubfs.h>
#define DISK_DRIVE_NAME "/mcubfs"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#elif defined(CONFIG_FILE_SYSTEM_FATFS)
#include <fs/fatfs.h>
#define DISK_DRIVE_NAME "/fatfs"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#elif defined(CONFIG_FILE_SYSTEM_FATFS_LFN)
#include <fs/fatfs.h>
#define DISK_DRIVE_NAME "/fatfs"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#elif defined(CONFIG_FILE_SYSTEM_FATFS_PETIT)
#include <fs/fatfs.h>
#define DISK_DRIVE_NAME "/fatfs"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#elif defined(CONFIG_FILE_SYSTEM_FATFS_FS)
#include <fs/fatfs.h>
#define DISK_DRIVE_NAME "/fatfs"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#elif defined(CONFIG_FILE_SYSTEM_FATFS_FS_LFN)
#include <fs/fatfs.h>
#define DISK_DRIVE_NAME "/fatfs"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#elif defined(CONFIG_FILE_SYSTEM_FATFS_FS_PETIT)
#include <fs/fatfs.h>
#define DISK_DRIVE_NAME "/fatfs"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#elif defined(CONFIG_FILE_SYSTEM_FATFS_FS_NFFS)
#include <fs/fatfs.h>
#define DISK_DRIVE_NAME "/fatfs"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#else	// defaults here for now until prj.conf is updated to FS I want to use
#define DISK_DRIVE_NAME "/default"
#define DISK_MOUNT_PT "/"DISK_DRIVE_NAME":"

#endif

/* 1000 msec = 1 sec */
#define SLEEP_TIME_MS   500

/* The devicetree node identifier for the "led0" alias. */
#define LED0_NODE DT_ALIAS(led1)

/* A build error on this line means your board is unsupported */
static const struct gpio_dt_spec led = GPIO_DT_SPEC_GET(LED0_NODE, gpios);

LOG_MODULE_REGISTER(main);

#define MAX_PATH 128
#define SOME_FILE_NAME "some.dat"
#define SOME_DIR_NAME "some"
#define SOME_REQUIRED_LEN MAX(sizeof(SOME_FILE_NAME), sizeof(SOME_DIR_NAME))

static int lsdir(const char *path);
#ifdef CONFIG_FS_SAMPLE_CREATE_SOME_ENTRIES

static bool createEntries(const char *base_path)
{
	char path[MAX_PATH];
	struct fs_file_t file;
	int base = strlen(base_path);

	fs_file_t_init(&file);

	if (base >= (sizeof(path) - SOME_REQUIRED_LEN)) {
		LOG_ERR("Not enough concatenation buffer to create file paths");
		return false;
	}

	LOG_INF("Creating some dir entries in %s", base_path);
	strncpy(path, base_path, sizeof(path));

	path[base++] = '/';
	path[base] = 0;
	strcat(&path[base], SOME_FILE_NAME);

	if (fs_open(&file, path, FS_O_CREATE) != 0) {
		LOG_ERR("Failed to create file %s", path);
		return false;
	}
	fs_close(&file);

	path[base] = 0;
	strcat(&path[base], SOME_DIR_NAME);

	if (fs_mkdir(path) != 0) {
		LOG_ERR("Failed to create dir %s", path);
		/* If code gets here, it has at least successes to create the
		 * file so allow function to return true.
		 */
	}
	return true;
}

#endif

static const char *disk_mount_pt = DISK_MOUNT_PT;

int main(void)
{
	int ret;
	bool led_state = true;

	printk("Running...\n");

	/* raw disk I/O */
	do {
		static const char *disk_pdrv = DISK_DRIVE_NAME;
		uint64_t memory_size_mb;
		uint32_t block_count;
		uint32_t block_size;
		// uint32_t erase_block_size;  // future use? 
		// uint32_t write_block_size;

		if (disk_access_init(disk_pdrv) != 0) {
			LOG_ERR("SD card initialization failed!\n");
			break;
		}

		if (disk_access_ioctl(disk_pdrv, DISK_IOCTL_GET_SECTOR_COUNT, &block_count)) {
			LOG_ERR("Unable to get block count!\n");
			break;
		}

		LOG_INF("Block count %u", block_count);
		// printf("Block count inside printf: %u", block_count);

		if (disk_access_ioctl(disk_pdrv, DISK_IOCTL_GET_SECTOR_SIZE, &block_size)) {
			LOG_ERR("Unable to get sector size!\n");
			break;
		}

		LOG_INF("Sector size %u\n", block_size);
	}
	while (0);

	if (!gpio_is_ready_dt(&led)) {
		return 0;
	}

	ret = gpio_pin_configure_dt(&led, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		return 0;
	}

	while (1) {
		ret = gpio_pin_toggle_dt(&led);
		if (ret < 0) {
			return 0;
		}

		led_state = !led_state;
		printf("LED state: %s\n", led_state ? "ON" : "OFF");
		k_msleep(SLEEP_TIME_MS);
	}
	return 0;
}
